import { freeze, produce } from 'immer';
import { useState } from 'react';
import { useUnmount } from './useUnmount';
export function createGlobalState(initialValue) {
    const store = {
        state: freeze(typeof initialValue === 'function' ? initialValue() : initialValue, true),
        setState(updater) {
            if (typeof updater === 'function') {
                store.state = produce(store.state, updater);
            }
            else {
                store.state = freeze(updater);
            }
            for (const update of store.updates) {
                update(store.state);
            }
        },
        updates: new Set(),
    };
    return Object.assign(() => {
        const [state, setState] = useState(store.state);
        if (!store.updates.has(setState)) {
            store.updates.add(setState);
        }
        useUnmount(() => {
            store.updates.delete(setState);
        });
        return [state, store.setState];
    }, {
        set: store.setState,
    });
}
